@@ -75,7 +75,7 @@ sub new {
         Name => $name,
     }, $type;
 
-    my $inccode = join '',map {$_->get_incregisters();} (values %{$this->{ParObjs}});
+    my $inccode = join '',map {$_->get_incregisters();} (sort values %{$this->{ParObjs}});
 
     # First, separate the code into an array of C fragments (strings),
     # variable references (strings starting with $) and
@@ -165,7 +165,7 @@ sub new {
 
     # Then, in this form, put it together what we want the code to actually do.
     print "SIZEPRIVS: ",(join ',',%$sizeprivs),"\n" if $::PP_VERBOSE;
-    $this->{Code} = "{".(join '',values %$sizeprivs).
+    $this->{Code} = "{".(join '',sort values %$sizeprivs).
        $coderef->get_str($this,[])
        ."}";
     $this->{Code};
@@ -251,7 +251,9 @@ sub get_str {
     my $bad  = $this->[1];
 
     my $str = "if ( \$PRIV(bvalflag) ) { PDL_COMMENT(\"** do 'bad' Code **\")\n";
+    $str .= "\n#define PDL_BAD_CODE\n";
     $str .= $bad->get_str($parent,$context);
+    $str .= "\n#undef PDL_BAD_CODE\n";
     $str .= "} else { PDL_COMMENT(\"** else do 'good' Code **\")\n";
     $str .= $good->get_str($parent,$context);
     $str .= "}\n";
@@ -272,7 +274,7 @@ sub new { my($type,$args,$sizeprivs,$parent) = @_;
 		print "SIZP $sizeprivs, $_\n" if $::PP_VERBOSE;
 		my $i = $parent->make_loopind($_);
 		$sizeprivs->{$i->[0]} =
-		  "register int __$i->[0]_size = \$PRIV(__$i->[0]_size);\n";
+		  "register PDL_Indx __$i->[0]_size = \$PRIV(__$i->[0]_size);\n";
 		print "SP :",(join ',',%$sizeprivs),"\n" if $::PP_VERBOSE;
 	}
 	return $this;
@@ -284,7 +286,7 @@ sub myprelude { my($this,$parent,$context) = @_;
 	push @$context, map {
 		$i = $parent->make_loopind($_);
 # Used to be $PRIV(.._size) but now we have it in a register.
-		$text .= "{PDL_COMMENT(\"Open $_\") register int $_;
+		$text .= "{PDL_COMMENT(\"Open $_\") register PDL_Indx $_;
 			for($_=0; $_<(__$i->[0]_size); $_++) {";
 		$i;
 	} @{$this->[0]};
@@ -455,52 +457,47 @@ sub myprelude {
     my $funcName = "readdata";
     $funcName = "writebackdata" if( $backcode );
 
-
-    my $no; my $no2=-1; my $no3=-1;
     my ($ord,$pdls) = $parent->get_pdls();
 
-'	PDL_COMMENT("THREADLOOPBEGIN")
- if(PDL->startthreadloop(&($PRIV(__pdlthread)),$PRIV(vtable)->'.$funcName.',
- 	__tr)) return;
-   do { register int __tind1=0,__tind2=0;
-        register int __tnpdls = $PRIV(__pdlthread).npdls;
-      register int __tdims1 = $PRIV(__pdlthread.dims[1]);
-      register int __tdims0 = $PRIV(__pdlthread.dims[0]);
-      register int *__offsp = PDL->get_threadoffsp(&$PRIV(__pdlthread));
- '.(join '',map {$no2++;
-            "register int __tinc0_".($no2)." = \$PRIV(__pdlthread).incs[$no2];"}
-	     @$ord).
-   (join '',map {$no3++;
-            "register int __tinc1_".($no3)." = \$PRIV(__pdlthread).incs[__tnpdls+$no3];"}
-	     @$ord).
-   (join '',map {"${_}_datap += __offsp[".(0+$no++)."];\n"}
- 		@$ord).'
-	for(__tind2=0; __tind2<__tdims1 ; __tind2++) {
-	 for(__tind1=0; __tind1<__tdims0 ; __tind1++) {
-	  PDL_COMMENT("This is the tightest threadloop. Make sure inside is optimal.")
-';
+    join( "\n	",
+	  '',
+	  'PDL_COMMENT("THREADLOOPBEGIN")',
+	  'if ( PDL->startthreadloop(&($PRIV(__pdlthread)),$PRIV(vtable)->'.$funcName.', __tr) ) return;
+           do { register PDL_Indx __tind1=0,__tind2=0;
+                register PDL_Indx __tnpdls = $PRIV(__pdlthread).npdls;
+                register PDL_Indx __tdims1 = $PRIV(__pdlthread.dims[1]);
+                register PDL_Indx __tdims0 = $PRIV(__pdlthread.dims[0]);
+                register PDL_Indx *__offsp = PDL->get_threadoffsp(&$PRIV(__pdlthread));',
+	  ( map { "register PDL_Indx __tinc0_${_} = \$PRIV(__pdlthread).incs[${_}];"} 0..$#{$ord}),
+	  ( map { "register PDL_Indx __tinc1_${_} = \$PRIV(__pdlthread).incs[__tnpdls+$_];"} 0.. $#{$ord}),
+	  ( map { $ord->[$_] ."_datap += __offsp[$_];"} 0..$#{$ord} ),
+	  'for( __tind2 = 0 ;
+                __tind2 < __tdims1 ;
+                __tind2++',
+	        ( map { "\t\t," . $ord->[$_] . "_datap += __tinc1_${_} - __tinc0_${_} * __tdims0"} 0..$#{$ord} ),
+             ')',
+	  '{
+	     for( __tind1 = 0 ;
+                  __tind1 < __tdims0 ;
+                  __tind1++',
+	          ( map { "\t\t," . $ord->[$_] . "_datap += __tinc0_${_}"} 0..$#{$ord}),
+	       ')',
+           '{  PDL_COMMENT("This is the tightest threadloop. Make sure inside is optimal.")'
+	);
 }
 
 # Should possibly fold out thread.dims[0] and [1].
 sub mypostlude {my($this,$parent,$context) = @_;
- my $no; my $no0; my $no1; my $no2; my $no3; my $no4; my $no5;
+
  my ($ord,$pdls) = $parent->get_pdls();
-'	PDL_COMMENT("THREADLOOPEND")
-	 '.(join '',map {"${_}_datap += __tinc0_".(0+$no0++).";\n"}
- 		@$ord).'
-	 } '
-	 .(join '',map {"${_}_datap += __tinc1_".(0+$no1++)."
-	     			     - __tinc0_".(0+$no2++)." *
-				       __tdims0;\n"}
- 		@$ord).'
-	} '.
-     (join '',map {"${_}_datap -= __tinc1_".(0+$no3++)." *
-     				  __tdims1;"}
- 		@$ord).'
- '.(join '',map {"${_}_datap -= __offsp[".(0+$no++)."];\n"}
- 		@$ord).'
-      } while(PDL->iterthreadloop(&$PRIV(__pdlthread),2));
- '
+ join( "\n	",
+       '',
+       'PDL_COMMENT("THREADLOOPEND")',
+       '}',
+       '}',
+       ( map { $ord->[$_] . "_datap -= __tinc1_${_} * __tdims1 + __offsp[${_}];"} 0..$#{$ord} ),
+       '} while(PDL->iterthreadloop(&$PRIV(__pdlthread),2));'
+     )
 }
 
 # Simple subclass of ComplexThreadLoop to implement writeback code
@@ -1200,7 +1197,7 @@ package PDL::PP::Code;
 # loops (array references, 1. item = variable.
 #
 sub separate_code {
-	$DB::single=1;
+   ## $DB::single=1;
     my ( $this, $code ) = @_;
 
     # First check for standard code errors:
