@@ -1,32 +1,66 @@
 ##############################################
+
+##############################################
+
 package PDL::PP::PdlParObj;
+
 use Carp;
-use SelfLoader;
-use PDL::Core;
 use PDL::Types;
 
-@ISA = qw/ SelfLoader /;
+# check for bad value support
+#
+use PDL::Config;
+my $usenan = $PDL::Config{BADVAL_USENAN} || 0;
 
-# need some mods in Types and Core for that
-# for (byte,short,ushort,long,float,double) {
-#   $Typemap{$_->name} = {  Ctype => $_->ctype,
-# 			  Cenum => $_->enum,
-# 			  Val => $_->val };
-# }
+our %Typemap = ();
+use PDL::Types ':All';
 
-%PDL::PP::PdlParObj::Typemap = ();
-my $type;
-for (['Byte',$PDL_B],
-      ['Short',$PDL_S],
-      ['Ushort',$PDL_US],
-      ['Long',$PDL_L],
-      ['Float',$PDL_F],
-      ['Double',$PDL_D]) {
-  $type = ($_->[0] =~ /Long/ ? 'int' : lc $_->[0]);
-  $Typemap{$type} = { Ctype => "PDL_$_->[0]",
-		      Cenum => ($type =~ /ushort/ ? "PDL_US" :
-		                   "PDL_".substr($_->[0],0,1)),
-		      Val => $_->[1]  };
+# build a typemap for our translation purposes
+# again from info in PDL::Types
+for my $typ (typesrtkeys) {
+  $Typemap{typefld($typ,'ppforcetype')} = {
+					  Ctype => typefld($typ,'ctype'),
+					  Cenum => typefld($typ,'sym'),
+					  Val =>   typefld($typ,'numval'),
+					 };
+}
+
+# Try to load Text::Balanced
+my $hasTB = 0;
+eval q{
+	use Text::Balanced;
+	$hasTB = 1;
+};
+
+# split regex $re separated arglist
+# but ignore bracket-protected bits
+# (i.e. text that is within matched brackets)
+# fallback to simple split if we can't find Text::Balanced
+my $prebrackreg = qr/^([^\(\{\[]*)/;
+sub splitprotected ($$) {
+  my ($re,$txt) = @_;
+  return split $re, $txt unless $hasTB;
+  return () if !defined $txt || $txt =~ /^\s*$/;
+  my ($got,$pre) = (1,'');
+  my @chunks = ('');
+  my $ct = 0; # infinite loop protection
+  while ($got && $txt =~ /[({\[]/ && $ct++ < 1000) {
+    # print "iteration $ct\n";
+    ($got,$txt,$pre) =
+      Text::Balanced::extract_bracketed($txt,'{}()[]',$prebrackreg);
+    my @partialargs = split $re, $pre, -1;
+    $chunks[-1] .= shift @partialargs if @partialargs;
+    push @chunks, @partialargs;
+    $chunks[-1] .= $got;
+  }
+  confess "possible infinite parse loop, splitting '$txt' "
+			   if $ct >= 1000;
+  my @partialargs = split $re, $txt, -1;
+  $chunks[-1] .= shift @partialargs if @partialargs;
+  push @chunks, @partialargs if @partialargs;
+  # print STDERR "args found: $#chunks\n";
+  # print STDERR "splitprotected $txt on $re: [",join('|',@chunks),"]\n";
+  return @chunks;
 }
 
 # null != [0]
@@ -40,21 +74,28 @@ for (['Byte',$PDL_B],
 
 1;
 
-__DATA__
+#__DATA__
 
+# need for $badflag is due to hacked get_xsdatapdecl() 
+# - this should disappear when (if?) things are done sensibly
+#
+my $typeregex = join '|', map {typefld($_,'ppforcetype')} typesrtkeys;
+our $pars_re = qr/^
+	\s*((?:$typeregex)[+]*|)\s*	# $1: first option
+	(?:
+	\[([^]]*)\]   	# $2: The initial [option] part
+	)?\s*
+	(\w+)          	# $3: The name
+	\(([^)]*)\)  		# $4: The indices
+/x;
 sub new {
-	my($type,$string,$number) = @_;
-	my $this = bless {Number => $number},$type;
-# Parse the parameter string
-	$string =~
-		/^
-		 \s*((?:byte|short|ushort|int|float|double)[+]*|)\s*	# $1: first option
-		 (?:
-			\[([^]]*)\]   	# $2: The initial [option] part
-	         )?\s*
-		 (\w+)          	# $3: The name
-		 \(([^)]*)\)  		# $4: The indices
-		/x or confess "Invalid pdl def $string\n";
+	my($type,$string,$number,$badflag) = @_;
+	$badflag ||= 0;
+	my $this = bless {Number => $number, BadFlag => $badflag},$type;
+	# Parse the parameter string. Note that the regexes for this match were
+	# originally defined here, but were moved to PDL::PP for FullDoc parsing.
+	$string =~ $pars_re
+		 or confess "Invalid pdl def $string (regex $typeregex)\n";
 	my($opt1,$opt2,$name,$inds) = ($1,$2,$3,$4);
 	map {$_ = '' unless defined($_)} ($opt1,$opt2,$inds); # shut up -w
 	print "PDL: '$opt1', '$opt2', '$name', '$inds'\n"
@@ -70,12 +111,12 @@ sub new {
 			and $this->{FlagCreateAlways}=1 or
 		/^t$/ and $this->{FlagTemp}=1 and $this->{FlagCreat}=1 and $this->{FlagW}=1 or
 		/^phys$/ and $this->{FlagPhys} = 1 or
-		/^((?:byte|short|ushort|int|float|double)[+]*)$/ and $this->{Type} = $1 and $this->{FlagTyped} = 1 or
+		/^((?:$typeregex)[+]*)$/ and $this->{Type} = $1 and $this->{FlagTyped} = 1 or
 		confess("Invalid flag $_ given for $string\n");
 	}
-	if($this->{FlagPhys}) {
-		# warn("Warning: physical flag not implemented yet");
-	}
+#	if($this->{FlagPhys}) {
+#		# warn("Warning: physical flag not implemented yet");
+#	}
 	if ($this->{FlagTyped} && $this->{Type} =~ s/[+]$// ) {
 	  $this->{FlagTplus} = 1;
 		}
@@ -155,7 +196,11 @@ sub getcreatedims {
 sub typeval {
   my $ctype = shift;
   my @match = grep {$Typemap{$_}->{Ctype} =~ /^$ctype$/} keys(%Typemap);
-  croak "unknown PDL type '$ctype'" if $#match < 0;
+  if ($#match < 0) {
+    use Data::Dumper;
+    print Dumper \%Typemap;
+    croak "unknown PDL type '$ctype'" ;
+  }
   return $Typemap{$match[0]}->{Val};
 }
 
@@ -176,10 +221,10 @@ sub ctype {
 
 # return the enum type for a parobj; it'd better be typed
 sub cenum {
-  my $this = shift;
-  croak "cenum: unknownn type"
-    unless defined($PDL::PP::PdlParObj::Typemap{$this->{Type}});
-  return $PDL::PP::PdlParObj::Typemap{$this->{Type}}->{Cenum};
+    my $this = shift;
+    croak "cenum: unknown type [" . $this->{Type} . "]"
+	unless defined($PDL::PP::PdlParObj::Typemap{$this->{Type}});
+    return $PDL::PP::PdlParObj::Typemap{$this->{Type}}->{Cenum};
 }
 
 sub get_nname{ my($this) = @_;
@@ -192,58 +237,70 @@ sub get_nnflag { my($this) = @_;
 
 
 # XXX There might be weird backprop-of-changed stuff for [phys].
-sub get_xsnormdimchecks { my($this) = @_;
-	my $pdl = $this->get_nname;
-	my $str = ""; my $ninds = 0+scalar(@{$this->{IndObjs}});
-	$str .= "if(!__creating[$this->{Number}]) {";
-	# Dimensional Promotion when number of dims is less than required:
-	#   Previous warning message now commented out.
-	$str .= "
-		if(($pdl)->ndims < $ninds) {
-                 ".join('', map {       
-		"if (($pdl)->ndims < $_ && ".$this->{IndObjs}[$_-1]->get_size()." <= 1)
-		".$this->{IndObjs}[$_-1]->get_size() ." = 1;\n"}
-                      (1..$ninds))."
-                /*      \$CROAK(\"Too few dimensions for argument \'$this->{Name}\'\\n\"); */
-		}
-	";
-# Now, the real check.
-	my $no = 0;
-	for(@{$this->{IndObjs}}) {
-		my $siz = $_->get_size();
-		my $dim = "($pdl)->dims[$no]";
-               my $ndims = "($pdl)->ndims";
-		$str .= "
-                 if($siz == -1 || ($ndims > $no && $siz == 1)) {
-			$siz = $dim;
-                 } else if($ndims > $no && $siz != $dim) {
-		  	if($dim == 1) {
-				/* Do nothing */ /* XXX Careful, increment? */
-			} else {
-				\$CROAK(\"Wrong dims\\n\");
-			}
-		  }
-		";
-		$no++;
-	}
-	if($this->{FlagPhys}) {
-		$str .= "PDL->make_physical(($pdl));";
-	}
-	$str .= "} else {";
-# We are creating this pdl.
-	if(!$this->{FlagCreat}) {
-		$str .= qq'\$CROAK("Cannot create non-output argument $this->{Name}!\\n");';
-	} else {
-		$str .= "int dims[".($ninds+1)."]; /* Use ninds+1 to avoid smart (stupid) compilers */";
-		$str .= join "",
-		   (map {"dims[$_] = ".$this->{IndObjs}[$_]->get_size().";"}
-		      0..$#{$this->{IndObjs}});
-		my $istemp = $this->{FlagTemp} ? 1 : 0;
-              $str .="\n PDL->thread_create_parameter(&\$PRIV(__pdlthread),$this->{Number},dims,$istemp);\n"
-	}
+#
+# Have changed code to assume that, if(!$this->{FlagCreat})
+# then __creating[] will == 0
+#  -- see make_redodims_thread() in ../PP.pm
+#
+sub get_xsnormdimchecks { 
+    my($this) = @_;
+    my $pdl   = $this->get_nname;
+    my $iref  = $this->{IndObjs};
+    my $ninds = 0+scalar(@$iref);
+
+    my $str = ""; 
+    $str .= "if(!__creating[$this->{Number}]) {\n" if $this->{FlagCreat};
+    
+    # Dimensional Promotion when number of dims is less than required:
+    #   Previous warning message now commented out,
+    #   which means we only need include the code if $ninds > 0
+    #
+    if ( $ninds > 0 ) {
+	$str .= "   if(($pdl)->ndims < $ninds) {\n" .
+	    join('', map { 
+		my $size = $iref->[$_-1]->get_size();      
+		"      if (($pdl)->ndims < $_ && $size <= 1) $size = 1;\n"
+		} (1..$ninds)) 
+# XXX why is this here, commented, and not removed? If re-inserted, be sure to use PDL_COMMENT
+##		."      /* \$CROAK(\"Too few dimensions for argument \'$this->{Name}\'\\n\"); */\n"
+		. "   }\n";
+    }
+
+    # Now, the real check.
+    my $no = 0;
+    for( @$iref ) {
+	my $siz = $_->get_size();
+	my $dim = "($pdl)->dims[$no]";
+	my $ndims = "($pdl)->ndims";
+	$str .= "   if($siz == -1 || ($ndims > $no && $siz == 1)) {\n" .
+	        "      $siz = $dim;\n" .
+		"   } else if($ndims > $no && $siz != $dim) {\n" .
+# XXX should these lines simply be removed? If re-inserted, be sure to use PDL_COMMENT
+#		"      if($dim == 1) {\n" .
+#		"         /* Do nothing */ /* XXX Careful, increment? */" .
+#		"      } else {\n" .
+		"      if($dim != 1) {\n" .
+                "         \$CROAK(\"Wrong dims\\n\");\n" .
+		"      }\n   }\n";
+	$no++;
+    } 
+
+    $str .= "PDL->make_physical(($pdl));\n" if $this->{FlagPhys};
+
+    if ( $this->{FlagCreat} ) { 
+	$str .= "} else {\n";
+	
+	# We are creating this pdl.
+	$str .= " PDL_Indx dims[".($ninds+1)."]; PDL_COMMENT(\"Use ninds+1 to avoid smart (stupid) compilers\")";
+	$str .= join "",
+	(map {"dims[$_] = ".$iref->[$_]->get_size().";"} 0 .. $#$iref);
+	my $istemp = $this->{FlagTemp} ? 1 : 0;
+	$str .="\n PDL->thread_create_parameter(&\$PRIV(__pdlthread),$this->{Number},dims,$istemp);\n";
 	$str .= "}";
-	$str
-}
+    }
+    return $str;
+    
+} # sub: get_xsnormdimchecks()
 
 sub get_incname {
 	my($this,$ind) = @_;
@@ -258,7 +315,7 @@ sub get_incdecls {
 	my($this) = @_;
 	if(scalar(@{$this->{IndObjs}}) == 0) {return "";}
 	(join '',map {
-		"PDL_Long ".($this->get_incname($_)).";";
+		"PDL_Indx ".($this->get_incname($_)).";";
 	} (0..$#{$this->{IndObjs}}) ) . ";"
 }
 
@@ -266,7 +323,7 @@ sub get_incregisters {
 	my($this) = @_;
 	if(scalar(@{$this->{IndObjs}}) == 0) {return "";}
 	(join '',map {
-		"register PDL_Long ".($this->get_incname($_))." = \$PRIV(".
+		"register PDL_Indx ".($this->get_incname($_))." = \$PRIV(".
 			($this->get_incname($_)).");\n";
 	} (0..$#{$this->{IndObjs}}) )
 }
@@ -298,8 +355,8 @@ sub do_access {
 	my $pdl = $this->{Name};
 # Parse substitutions into hash
 	my %subst = map
-	 {/^\s*(\w+)\s*=>\s*(\w*)\s*$/ or confess "Invalid subst $_\n"; ($1,$2)}
-	 	split ',',$inds;
+	 {/^\s*(\w+)\s*=>\s*(\S*)\s*$/ or confess "Invalid subst $_\n"; ($1,$2)}
+	 	splitprotected ',',$inds;
 # Generate the text
 	my $text;
 	$text = "(${pdl}_datap)"."[";
@@ -312,7 +369,7 @@ sub do_access {
 	if(scalar(keys %subst) != 0) {
 		confess("Substitutions left: ".(join ',',keys %subst)."\n");
 	}
-       return "$text /* ACCESS($access) */\n";
+       return "$text PDL_COMMENT(\"ACCESS($access)\") ";
 }
 
 sub has_dim {
@@ -376,16 +433,38 @@ sub do_indterm { my($this,$pdl,$ind,$subst,$context) = @_;
                "PP_INDTERM(".$this->{IndObjs}[$ind]->get_size().", $index))");
 }
 
-sub get_xsdatapdecl { my($this,$genlooptype,$asgnonly) = @_;
-	my $type; my $pdl = $this->get_nname; my $flag = $this->get_nnflag;
-		      my $name = $this->{Name};
-	$type = $this->ctype($genlooptype) if defined $genlooptype;
-	my $declini = ($asgnonly ? "" : "\t$type *");
-	my $cast = ($type ? "($type *)" : "");
+# XXX hacked to create a variable containing the bad value for 
+# this piddle. 
+# This is a HACK (Doug Burke 07/08/00)
+# XXX
+#
+sub get_xsdatapdecl { 
+    my($this,$genlooptype,$asgnonly) = @_;
+    my $type; 
+    my $pdl = $this->get_nname; 
+    my $flag = $this->get_nnflag;
+    my $name = $this->{Name};
+    $type = $this->ctype($genlooptype) if defined $genlooptype;
+    my $declini = ($asgnonly ? "" : "\t$type *");
+    my $cast = ($type ? "($type *)" : "");
 # ThreadLoop does this for us.
 #	return "$declini ${name}_datap = ($cast((${_})->data)) + (${_})->offs;\n";
-	return "$declini ${name}_datap = ($cast(PDL_REPRP_TRANS($pdl,$flag)));
-		$declini ${name}_physdatap = ($cast($pdl->data));
-	\n";
+    
+    my $str = "$declini ${name}_datap = ($cast(PDL_REPRP_TRANS($pdl,$flag)));\n" .
+	"$declini ${name}_physdatap = ($cast($pdl->data));\n";
+
+    # assuming we always need this 
+    # - may not be true - eg if $asgnonly ??
+    # - not needed for floating point types when using NaN as bad values
+    if ( $this->{BadFlag} and $type and 
+	 ( $usenan == 0 or $type !~ /^PDL_(Float|Double)$/ ) ) {
+	my $cname = $type; $cname =~ s/^PDL_//;
+	$str .= "\t$type   ${name}_badval = 0;\n";
+	$str .= "\tPDL_Anyval  ${name}_anyval_badval = PDL->get_pdl_badvalue($pdl);\n";
+	$str .= "\tANYVAL_TO_CTYPE(${name}_badval, ${type}, ${name}_anyval_badval);\n";
+    }	
+
+    return "$str\n";
 }
 
+1;
