@@ -64,8 +64,14 @@ Copyright 1984, 1987, 1989, 1995 by Stephen L. Moshier
 
 /* For PDL, use system defaults where possible */
 #include <math.h>
-#if !defined(WIN32) && !defined(_WIN32)
+#if !defined(WIN32) && !defined(_WIN32) && !defined(__APPLE__)
+/* values.h is gone on OpenBSD(?) and depracated on GNU systems */
+/* can we use values.h on all UN*X systems? */
+#if defined __GNUC__
+#include <limits.h>
+#else
 #include <values.h>
+#endif   /* __GNUC__ */
 #endif
 #if defined(_WIN32) || defined(WIN32)
 #include <float.h>
@@ -73,19 +79,26 @@ Copyright 1984, 1987, 1989, 1995 by Stephen L. Moshier
 #endif
 
 /* Now include system-specific stuff */
+
+/* Look for system quiet_nan function */
 #if defined __sun && ! defined __GNUC__
 #include <sunmath.h>
 #include <ieeefp.h>
 #include <float.h>
 #define NANARG 1L
+#define NANARG_SIGNATURE long n
 #endif
-#if defined __alpha
+#if defined __alpha && ! defined __linux
 #include <float.h>
 #include <nan.h>
 #endif
 #ifndef NANARG
 #define NANARG
+#define NANARG_SIGNATURE
 #endif
+
+/* Redefine nan so PDL doesn't die when we see one.
+   OK, nasty, but means the C-code is still as in the original */
 #define nan() quiet_nan(NANARG)
 
 /* Constant definitions for math error conditions */
@@ -143,7 +156,7 @@ int mtherr();
 extern int merror;
 
 #ifdef MY_QUIET_NAN
-extern double quiet_nan();
+extern double quiet_nan(NANARG_SIGNATURE);
 #endif
 #ifdef MY_INFINITY
 extern double infinity();
