@@ -38,8 +38,8 @@ comment q|
 |;
 
 act q|
-	$x = zeroes 5,5; # 5x5 matrix
-	output $x;
+	$a = zeroes 5,5; # 5x5 matrix
+	output $a;
 |;
 
 act q|
@@ -50,21 +50,21 @@ act q|
 |;
 
 act q|
-	$x ++;      # Operators like increment work..
-	output $x;
+	$a ++;      # Operators like increment work..
+	output $a;
 |;
 
 act q|
 	# xvals and yvals (yes, there is also zvals...)
-	# give you ndarrays which give the coordinate value.
-	$y = xvals $x;
-	output $y;
+	# give you piddles which give the coordinate value.
+	$b = xvals $a;
+	output $b;
 |;
 
 act q|
 	# So you can do things like
-	$y = $x + 0.1 * xvals($x) + 0.01 * yvals($x);
-	output $y;
+	$b = $a + 0.1 * xvals($a) + 0.01 * yvals($a);
+	output $b;
 |;
 
 act q|
@@ -76,41 +76,41 @@ act q|
 
 act q|
 	# You can also take slices:
-	output $y;
-	output $y->slice(":,2:3");  # rows 2 and 3
+	output $b;
+	output $b->slice(":,2:3");  # rows 2 and 3
 |;
 act q|
-	output $y->slice("2:3,:");  # or columns 2 and 3
+	output $b->slice("2:3,:");  # or columns 2 and 3
 |;
 
 act q|
-	output $y;
-	output $y->diagonal(0,1),"\n"; # 0 and 1 are the dimensions
+	output $b;
+	output $b->diagonal(0,1),"\n"; # 0 and 1 are the dimensions
 |;
 
 act q|
 	# One of the really nifty features is that the
 	# slices are actually references back to the original
-	# ndarray:
-	$diag = $y->diagonal(0,1);
-	output $y;
+	# piddle:
+	$diag = $b->diagonal(0,1);
+	output $b;
 	output $diag,"\n";
 	$diag+=100;
 	output "AFTER:\n";
 	output $diag,"\n";
-	output "Now, guess what \$y looks like?\n";
+	output "Now, guess what \$b looks like?\n";
 |;
 
 act q|
 	# Yes, it has changed:
-	output $y;
+	output $b;
 |;
 
 act q|
 	# Another example (we only modify elements 0,2 and 4 of
         # each row):
-	$t = $y->slice("0:4:2"); $t += 50;
-	output $y;
+	$t = $b->slice("0:4:2"); $t += 50;
+	output $b;
 |;
 
 act q|
@@ -118,35 +118,35 @@ act q|
 	# and PDL::Slices - we can't show you all but here are some
 	# examples:
 
-	output $y;
-	output $y->sum, "\n";
-	output $y->sumover,"\n"; # Only over first dim.
+	output $b;
+	output $b->sum, "\n";
+	output $b->sumover,"\n"; # Only over first dim.
 |;
 
 act q|
-	output $y->transpose;
-	output $y->minimum,"\n"; # over first dim.
-	output $y->min,"\n";
+	output $b->xchg(0,1);
+	output $b->minimum,"\n"; # over first dim.
+	output $b->min,"\n";
 |;
 
 act q|
-	output $y->random;
+	output $b->random;
 |;
 
 act q|
 	# Here are some more advanced tricks for selecting
 	# parts of 1-D vectors:
-	$x = (xvals 12)/3;
-	$i = which(sin($x) > 0.5);   # Indices of those sines > 0.5
-	output $x,"\n";
+	$a = (xvals 12)/3;
+	$i = which(sin($a) > 0.5);   # Indices of those sines > 0.5
+	output $a,"\n";
 	output $i,"\n";
-	output $x->index($i),"\n";
+	output $a->index($i),"\n";
              # and we can have the effect of the last command in one
              # go using 'where' instead of 'which' and 'index' as in
-        output $x->where(sin($x) > 0.5),"\n";
+        output $a->where(sin($a) > 0.5),"\n";
              # and finally take the sin of these elements
              # (to show that these are indeed the correct ones)
-	output sin($x->index($i)),"\n";
+	output sin($a->index($i)),"\n";
 |;
 
 comment q|
