@@ -5,15 +5,31 @@ PDL::Graphics::TriD -- PDL 3D interface
 =head1 SYNOPSIS
 
  use PDL::Graphics::TriD;
-
- # After each graph, let the user rotate is and
+ 
+ # Generate a somewhat interesting sequence of points:
+ $t = sequence(100)/10;
+ $x = sin($t); $y = cos($t), $z = $t;
+ $coords = cat($x, $y, $z)->xchg(0,1);
+ $r = cos(2*$t); $g = sin($t); $b = $t;
+ $colors = cat($r, $g, $b)->xchg(0,1);
+ 
+ # After each graph, let the user rotate and
  # wait for him to press 'q', then make new graph
  line3d($coords);       # $coords = (3,n,...)
  line3d($coords,$colors);  # $colors = (3,n,...)
  line3d([$x,$y,$z]);
- imagrgb([$r,$g,$b]);
- lattice3d([$x,$y,$z]); # 2-d piddles
+ 
+ # Generate a somewhat interesting sequence of surfaces
+ $surf1 = (rvals(100, 100) / 50)**2 + sin(xvals(100, 100) / 10);
+ $surf2 = sqrt(rvals(zeroes(50,50))/2);
+ $x = sin($surface); $y = cos($surface), $z = $surface;
+ $coords = cat($x, $y, $z)->xchg(0,1);
+ $r = cos(2*$surface); $g = sin($surface); $b = $surface;
+ $colors = cat($r, $g, $b)->xchg(0,1);
+ imagrgb([$r,$g,$b]);     # 2-d piddles
+ lattice3d([$surf1]);
  points3d([$x,$y,$z]);
+ spheres3d([$x,$y,$z]);  # preliminary implementation
 
  hold3d(); # the following graphs are on top of each other and the previous
  line3d([$x,$y,$z]);
@@ -37,6 +53,7 @@ Points, lines and surfaces (among other objects) are supported.
 With OpenGL, it is easy to manipulate the resulting 3D objects
 with the mouse in real time - this helps data visualization a lot.
 
+=for comment
 With VRML, you can generate objects for everyone to see with e.g.
 Silicon Graphics' Cosmo Player. You can find out more about VRML
 at C<http://vrml.sgi.com/> or C<http://www.vrml.org/>
@@ -61,7 +78,7 @@ OpenGL
 OpenGL but off-line (pixmap) rendering and writing to
 a graphics file.
 
-=item VRML
+=item VRML (I< Not available this release >)
 
 VRML objects rendering. This writes a VRML file describing the
 scene. This VRML file can then be read with  a browser.
@@ -93,13 +110,13 @@ interpretation.
 
 The alternative syntaxes for specifying a set of coordinates (or colors) are
 
-	$piddle                             # MUST have 3 as first dim.
-	[$piddle]
-	[$piddle1,$piddle2]
-	[$piddle1,$piddle2,$piddle3]
-	[CONTEXT,$piddle]
-	[CONTEXT,$piddle1,$piddle2]
-	[CONTEXT,$piddle1,$piddle2,$piddle3]
+   $piddle                             # MUST have 3 as first dim.
+  [$piddle]
+  [$piddle1,$piddle2]
+  [$piddle1,$piddle2,$piddle3]
+  [CONTEXT,$piddle]
+  [CONTEXT,$piddle1,$piddle2]
+  [CONTEXT,$piddle1,$piddle2,$piddle3]
 
 where C<CONTEXT> is a string describing in which context you wish these
 piddles to be interpreted. Each routine specifies a default context
@@ -113,39 +130,142 @@ The following contexts are currently supported:
 
 =item SURF2D
 
-A 2-D lattice. [$piddle] is interpreted as the Z coordinate over
+A 2-D lattice. C< [$piddle] > is interpreted as the Z coordinate over
 a lattice over the first dimension. Equivalent to
-[$piddle->xvals, $piddle->yvals, $piddle].
+C<< [$piddle->xvals, $piddle->yvals, $piddle] >>.
 
 =item POLAR2D
 
-A 2-D polar coordinate system. [$piddle] is interpreted as the
+A 2-D polar coordinate system. C< [$piddle] > is interpreted as the
 z coordinate over theta and r (theta = the first dimension of the piddle).
 
 =item COLOR
 
-A set of colors. [$piddle] is interpreted as grayscale color
-(equivalent to [$piddle,$piddle,$piddle]).
+A set of colors. C< [$piddle] > is interpreted as grayscale color
+(equivalent to C< [$piddle,$piddle,$piddle] >).
 
 =item LINE
 
-A line made of 1 or 2 coordinates. [$piddle] is interpreted as
-[$piddle->xvals,$piddle,0]. [$piddle1,$piddle2] is interpreted as
-[$piddle1,$piddle2,$piddle1->xvals].
+A line made of 1 or 2 coordinates. C< [$piddle] > is interpreted as
+C<< [$piddle->xvals,$piddle,0] >>. C< [$piddle1,$piddle2] > is interpreted as
+C<< [$piddle1,$piddle2,$piddle1->xvals] >>.
 
 =back
 
 What makes contexts useful is that if you want to plot points
 instead of the full surface you plotted with
 
-	imag3d([$zcoords]);
+  imag3d([$zcoords]);
 
 you don't need to start thinking about where to plot the points:
 
-	points3d([SURF2D,$zcoords]);
+  points3d([SURF2D,$zcoords]);
 
 will do exactly the same.
 
+=head2 Wrapping your head around 3d surface specifications
+
+Let's begin by thnking about how you might make a 2d data plot.
+If you sampled your data at regular intervals, you would have
+a time serires y(t) = (y0, y1, y2, ...).  You could plot y vs t
+by computing t0 = 0, t1 = dt, t2 = 2 * dt, and then plotting
+(t0, y0), (t1, y1), etc.
+
+Next suppose that you measured x(t) and y(t).  You can still
+plot y vs t, but you can also plot y vs x by plotting (x0, y0),
+(x1, y1), etc.  The x-values don't have to increase monotonically:
+they could back-track on each other, for example, like the
+latitude and longitude of a boat on a lake.  If you use plplot,
+you would plot this data using
+C<< $pl->xyplot($x, $y, PLOTTYPE => 'POINTS') >>.
+
+Good.  Now let's add a third coordinate, z(t).  If you actually
+sampled x and y at regular intervals, so that x and y lie on a
+grid, then you can construct a grid for z(x, y), and you would
+get a surface.  This is the situation in which you would use
+C<mesh3d([$surface])>.
+
+Of course, your data is not required to be regularly gridded.
+You could, for example, be measuring the flight path of a bat
+flying after mosquitos, which could be wheeling and arching
+all over the space.  This is what you might plot using 
+C<line3d([$x, $y, $z])>.  You could plot the trajectories of
+multiple bats, in which case C<$x>, C<$y>, and C<$z> would have
+multiple columns, but in general you wouldn't expect them to be
+coordinated.
+
+Finally, imagine that you have an air squadron flying in
+formation.  Your (x, y, z) data is not regularly gridded, but
+the (x, y, z) data for each plane should be coordinated and
+we can imagine that their flight path sweep out a surface.
+We could draw this data using C<line3d([$x, $y, $z])>, where
+each column in the variables corresponds to a different plane,
+but it would also make sense to draw this data using
+C<mesh3d([$x, $y, $z])>, since the planes' proximity to each
+other should be fairly consistent.  In other words, it makes
+sense to think of the planes as sweeping out a coordinated
+surface, which C<mesh3d> would draw for you, whereas you would
+not expect the trajectories of the various bats to describe a
+meaningful surface (unless you're into fractals, perhaps).
+
+ #!/usr/bin/perl
+
+ use PDL;
+ use PDL::Graphics::TriD;
+
+ # Draw out a trajectory in three-space
+ $t = sequence(100)/10;
+ $x = sin($t); $y = cos($t); $z = $t;
+
+ # Plot the trajectory as (x(t), y(t), z(t))
+ print "using line3d to plot a trajectory (press q when you're done twiddling)\n";
+ line3d [$x,$y,$z];
+
+ # If you give it a single piddle, it expects
+ # the data to look like
+ # ((x1, y1, z1), (x2, y2, z2), ...)
+ # which is why we have to do the exchange:
+ $coords = cat($x, $y, $z)->xchg(0,1);
+ print "again, with a different coordinate syntax (press q when you're done twiddling)\n";
+ line3d $coords;
+
+ # Draw a regularly-gridded surface:
+ $surface = sqrt(rvals(zeroes(50,50))/2);
+ print "draw a mesh of a regularly-gridded surface using mesh3d\n";
+ mesh3d [$surface];
+ print "draw a regularly-gridded surface using imag3d\n";
+ imag3d [$surface], {Lines=>0};
+
+ # Draw a mobius strip:
+ $two_pi = 8 * atan2(1,1);
+ $t = sequence(50) / 50 * $two_pi;
+ # We want two paths:
+ $mobius1_x = cos($t) + 0.5 * sin($t/2);
+ $mobius2_x = cos($t);
+ $mobius3_x = cos($t) - 0.5 * sin($t/2);
+ $mobius1_y = sin($t) + 0.5 * sin($t/2);
+ $mobius2_y = sin($t);
+ $mobius3_y = sin($t) - 0.5 * sin($t/2);
+ $mobius1_z = $t - $two_pi/2;
+ $mobius2_z = zeroes($t);
+ $mobius3_z = $two_pi/2 - $t;
+
+ $mobius_x = cat($mobius1_x, $mobius2_x, $mobius3_x);
+ $mobius_y = cat($mobius1_y, $mobius2_y, $mobius3_y);
+ $mobius_z = cat($mobius1_z, $mobius2_z, $mobius3_z);
+
+ $mobius_surface = cat($mobius_x, $mobius_y, $mobius_z)->mv(2,0);
+
+ print "A mobius strip using line3d one way\n";
+ line3d $mobius_surface;
+ print "A mobius strip using line3d the other way\n";
+ line3d $mobius_surface->xchg(1,2);
+ print "A mobius strip using mesh3d\n";
+ mesh3d $mobius_surface;
+ print "The same mobius strip using imag3d\n";
+ imag3d $mobius_surface, {Lines => 0};
+
+
 =head1 SIMPLE ROUTINES
 
 Because using the whole object-oriented interface for doing
@@ -169,11 +289,11 @@ routines are supported:
 
 Example:
 
- perldl> line3d [sqrt(rvals(zeroes(50,50))/2)]
+ pdl> line3d [sqrt(rvals(zeroes(50,50))/2)]
  - Lines on surface
- perldl> line3d [$x,$y,$z]
+ pdl> line3d [$x,$y,$z]
  - Lines over X, Y, Z
- perldl> line3d $coords
+ pdl> line3d $coords
  - Lines over the 3D coordinates in $coords.
 
 Note: line plots differ from mesh plots in that lines
@@ -197,7 +317,7 @@ contexts and options
 
 Example:
 
- perldl> imag3d [sqrt(rvals(zeroes(50,50))/2)], {{Lines=>0};
+ pdl> imag3d [sqrt(rvals(zeroes(50,50))/2)], {Lines=>0};
 
  - Rendered image of surface
 
@@ -219,7 +339,7 @@ contexts and options
 
 Example:
 
- perldl> mesh3d [sqrt(rvals(zeroes(50,50))/2)]
+ pdl> mesh3d [sqrt(rvals(zeroes(50,50))/2)]
 
  - mesh of surface
 
@@ -251,17 +371,40 @@ alias for mesh3d
 
 Example:
 
- perldl> points3d [sqrt(rvals(zeroes(50,50))/2)];
+ pdl> points3d [sqrt(rvals(zeroes(50,50))/2)];
  - points on surface
 
 See module documentation for more information on
 contexts and options
 
+=head2 spheres3d
+
+=for ref
+
+3D spheres plot (preliminary implementation)
+
+=for usage
+
+ spheres3d piddle(3), {OPTIONS}
+ spheres3d [piddle,...], {OPTIONS}
+
+=for example
+
+Example:
+
+ pdl> spheres3d ndcoords(10,10,10)->clump(1,2,3)  
+ 
+ - lattice of spheres at coordinates on 10x10x10 grid
+
+This is a preliminary implementation as a proof of
+concept.  It has fixed radii for the spheres being
+drawn and no control of color or transparency.
+
 =head2 imagrgb
 
 =for ref
 
-2D TrueColor Image plot
+2D RGB image plot (see also imag2d)
 
 =for usage
 
@@ -277,14 +420,14 @@ ways one might want to do this.
 
 e.g.
 
- perldl> $a=sqrt(rvals(zeroes(50,50))/2)
- perldl> imagrgb [0.5*sin(8*$a)+0.5,0.5*cos(8*$a)+0.5,0.5*cos(4*$a)+0.5]
+ pdl> $a=sqrt(rvals(zeroes(50,50))/2)
+ pdl> imagrgb [0.5*sin(8*$a)+0.5,0.5*cos(8*$a)+0.5,0.5*cos(4*$a)+0.5]
 
 =head2 imagrgb3d
 
 =for ref
 
-2D TrueColor Image plot as an object inside a 3D space
+2D RGB image plot as an object inside a 3D space
 
 =for usage
 
@@ -300,7 +443,7 @@ The default is [[0,0,0],[1,0,0],[1,1,0],[0,1,0]].
 
 e.g.
 
- perldl> imagrgb3d $colors, {Points => [[0,0,0],[1,0,0],[1,0,1],[0,0,1]]};
+ pdl> imagrgb3d $colors, {Points => [[0,0,0],[1,0,0],[1,0,1],[0,0,1]]};
  - plot on XZ plane instead of XY.
 
 =head2 grabpic3d
@@ -364,9 +507,9 @@ user explicitly presses 'q'.
  nokeeptwiddling3d();
  $o = imag3d($c);
  while(1) {
-	$c .= nextfunc($c);
-	$o->data_changed();
-	twiddle3d();		# animate one step, then return.
+ 	$c .= nextfunc($c);
+ 	$o->data_changed();
+ 	twiddle3d();		# animate one step, then return.
  }
 
 =head2 twiddle3d
@@ -420,10 +563,18 @@ an object like
 
 	$a = new PDL::Graphics::TriD::Scale($x,$y,$z);
 
+=head2 PDL::Graphics::TriD::LineStrip
+
+This is just a line or a set of lines. The arguments are 3 1-or-more-D
+piddles which describe the vertices of a continuous line and an 
+optional color piddle (which is 1-D also and simply
+defines the color between red and blue. This will probably change).
+
 =head2 PDL::Graphics::TriD::Lines
 
 This is just a line or a set of lines. The arguments are 3 1-or-more-D
-piddles and an optional color piddle (which is 1-D also and simply
+piddles where each contiguous pair of vertices describe a line segment 
+and an optional color piddle (which is 1-D also and simply
 defines the color between red and blue. This will probably change).
 
 =head2 PDL::Graphics::TriD::Image
@@ -460,7 +611,7 @@ man page.
 This is a special class: in order to obtain a new viewport, you
 need to have an earlier viewport on hand. The usage is:
 
-	$new_vp = $old_vp->new_viewport($x0,$y0,$x1,$y1);
+  $new_vp = $old_vp->new_viewport($x0,$y0,$x1,$y1);
 
 where $x0 etc are the coordinates of the upper left and lower right
 corners of the new viewport inside the previous (relative
@@ -469,21 +620,48 @@ to the previous viewport in the (0,1) range.
 Every implementation-level window object should implement the new_viewport
 method.
 
+=head1 EXAMPLE SCRIPT FOR VARIOUS 
+
 =cut
 
 #KGB: NEEDS DOCS ON COMMON OPTIONS!!!!!
 
+# List of global variables
+# 
+# $PDL::Graphics::TriD::offline
+# $PDL::Graphics::TriD::Settings 
+# $PDL::Graphics::TriD::verbose
+# $PDL::Graphics::TriD::keeptwiddling
+# $PDL::Graphics::TriD::hold_on
+# $PDL::Graphics::TriD::curgraph
+# $PDL::Graphics::TriD::cur
+# $PDL::Graphics::TriD::create_window_sub
+# $PDL::Graphics::TriD::current_window
+# 
+# '
 
 package PDL::Graphics::TriD::Basic;
 package PDL::Graphics::TriD;
+
+use PDL::Exporter;
+use PDL::Core '';  # barf
+use vars qw/@ISA @EXPORT_OK %EXPORT_TAGS/;
+@ISA = qw/PDL::Exporter/;
+@EXPORT_OK = qw/imag3d_ns imag3d line3d mesh3d lattice3d points3d
+  spheres3d describe3d imagrgb imagrgb3d hold3d release3d
+  keeptwiddling3d nokeeptwiddling3d
+  twiddle3d grabpic3d tridsettings/;
+%EXPORT_TAGS = (Func=>[@EXPORT_OK]);
+
+#use strict;
+use PDL::Graphics::TriD::Object;
+use PDL::Graphics::TriD::Window;
+use PDL::Graphics::TriD::ViewPort;
 use PDL::Graphics::TriD::Graph;
 use PDL::Graphics::TriD::Quaternion;
-# use PDL::Graphics::TriD::ArcBall;
-# use PDL::Graphics::TriD::SimpleScaler;
-# use PDL::Graphics::TriD::Control3D;
-use  PDL::Graphics::TriD::Objects;
+use PDL::Graphics::TriD::Objects;
 use PDL::Graphics::TriD::Rout;
-use PDL::Core '';  # barf
+
 
 # Then, see which display method are we using:
 
@@ -491,13 +669,15 @@ BEGIN {
 	my $dev;
 	$dev ||= $::PDL::Graphics::TriD::device; # First, take it from this variable.
 	$dev ||= $::ENV{PDL_3D_DEVICE};
-	if(!defined $dev) {
-#		warn "Default PDL 3D device is GL (OpenGL):
-#Set PDL_3D_DEVICE=GL in your environment in order not to see this warning.
-#You must have OpenGL or Mesa installed and the PDL::Graphics::OpenGL extension
-#compiled. Otherwise you will get strange warnings.";
-		$dev = $^O =~ /win32/i ? "VRML" : "GL";
-	}
+
+        if(!defined $dev) {
+#            warn "Default PDL 3D device is GL (OpenGL):
+# Set PDL_3D_DEVICE=GL in your environment in order not to see this warning.
+# You must have OpenGL or Mesa installed and the PDL::Graphics::OpenGL extension
+# compiled. Otherwise you will get strange warnings.";
+
+           $dev = "GL";  # default GL works on all platforms now
+        }
 	my $dv;
 # The following is just a sanity check.
 	for($dev) {
@@ -505,21 +685,17 @@ BEGIN {
 		(/^GL$/  and $dv="PDL::Graphics::TriD::GL") or
 		(/^GLpic$/  and $dv="PDL::Graphics::TriD::GL" and $PDL::Graphics::TriD::offline=1) or
 		(/^VRML$/  and $dv="PDL::Graphics::TriD::VRML" and $PDL::Graphics::TriD::offline=1) or
-		(die "Invalid PDL 3D device '$_' specified!");
+		(barf "Invalid PDL 3D device '$_' specified!");
 	}
 	my $mod = $dv;
 	$mod =~ s|::|//|g;
+	print "dev = $dev mod=$mod\n" if($verbose);
+ 
 	require "$mod.pm";
 	$dv->import;
+        my $verbose;
 }
 
-use PDL::Exporter;
-@ISA = qw/PDL::Exporter/;
-@EXPORT_OK = qw/imag3d_ns imag3d line3d mesh3d lattice3d points3d
-	describe3d imagrgb imagrgb3d hold3d release3d
-	keeptwiddling3d nokeeptwiddling3d
-	twiddle3d grabpic3d tridsettings/;
-%EXPORT_TAGS = (Func=>[@EXPORT_OK]);
 
 # currently only used by VRML backend
 sub tridsettings {return $PDL::Graphics::TriD::Settings}
@@ -530,13 +706,13 @@ sub realcoords {
 	my($type,$c) = @_;
 	if(ref $c ne "ARRAY") {
 		if($c->getdim(0) != 3) {
-			die "If one piddle given for coordinate, must be (3,...) or have default interpretation";
+			barf "If one piddle given for coordinate, must be (3,...) or have default interpretation";
 		}
 		return $c ;
 	}
 	if(!ref $c->[0]) {$type = shift @$c}
 	if($#$c < 0 || $#$c>2) {
-		die "Must have 1..3 array members for coordinates";
+		barf "Must have 1..3 array members for coordinates";
 	}
 	if($#$c == 0 and $type =~ /^SURF2D$/) {
 		# surf2d -> this is z axis
@@ -555,7 +731,13 @@ sub realcoords {
 	}
 	# XXX
 	if($#$c != 2) {
-		die("Must have 3 coordinates if no interpretation (here '$type')");
+		barf("Must have 3 coordinates if no interpretation (here '$type')");
+	}
+	# allow a constant (either pdl or not) to be introduced in one dimension
+        foreach(0..2){  
+	  if(ref($c->[$_]) ne "PDL" or $c->[$_]->nelem==1){
+	    $c->[$_] = $c->[$_]*(PDL->ones($c->[($_+1)%3]->dims));
+	  }
 	}
 	my $g = PDL->null;
 	&PDL::Graphics::TriD::Rout::combcoords(@$c,$g);
@@ -570,8 +752,11 @@ sub objplotcommand {
 }
 
 sub checkargs {
-	if(ref $_[$#_] eq "HASH") {
+	if(ref $_[$#_] eq "HASH" and $PDL::Graphics::TriD::verbose) {
+
+	  print "enter checkargs \n";
 		for([KeepTwiddling,\&keeptwiddling3d]) {
+		  print "checkargs >$_<\n";
 			if(defined $_[$#_]{$_->[0]}) {
 				&{$_->[1]}(delete $_[$#_]{$_->[0]});
 			}
@@ -596,13 +781,17 @@ sub PDL::twiddle3d {
 sub graph_object {
 	my($obj) = @_;
 	if(!defined $obj or !ref $obj) {
-		die("Invalid object to TriD::graph_object");
+		barf("Invalid object to TriD::graph_object");
 	}
+	print "graph_object: calling get_new_graph\n" if($PDL::debug_trid);
 	my $g = get_new_graph();
+	print "graph_object: back from get_new_graph\n" if($PDL::debug_trid);
+
 	my $name = $g->add_dataseries($obj);
 	$g->bind_default($name);
 	$g->scalethings();
 	print "ADDED TO GRAPH: '$name'\n" if $PDL::Graphics::TriD::verbose;
+
 	twiddle_current();
 	return $obj;
 }
@@ -625,9 +814,10 @@ sub PDL::imagrgb {
 	my (@data) = @_; &checkargs;
 	my $win = PDL::Graphics::TriD::get_current_window();
 	my $imag = new PDL::Graphics::TriD::Image(@data);
+
 	$win->clear_viewports();
-	my $vp = $win->new_viewport(0,0,1,1);
-	$vp->add_object($imag);
+
+	$win->current_viewport()->add_object($imag);
 	$win->twiddle();
 }
 
@@ -635,8 +825,18 @@ sub PDL::imagrgb {
 
 # Call: line3d([$x,$y,$z],[$color]);
 *line3d=\&PDL::line3d;
-sub PDL::line3d { &checkargs;
-	&graph_object(new PDL::Graphics::TriD::Lines(@_));
+sub PDL::line3d { 
+    &checkargs;
+    my $obj = new PDL::Graphics::TriD::LineStrip(@_);
+    print "line3d: object is $obj\n" if($PDL::debug_trid);
+    &graph_object($obj);
+}
+
+*contour3d=\&PDL::contour3d;
+sub PDL::contour3d { 
+#  &checkargs;
+  require PDL::Graphics::TriD::Contours;
+  &graph_object(new PDL::Graphics::TriD::Contours(@_));
 }
 
 # XXX Should enable different positioning...
@@ -656,6 +856,18 @@ sub PDL::imag3d { &checkargs;
 	&graph_object(new PDL::Graphics::TriD::SLattice_S(@_));
 }
 
+####################################################################
+################ JNK 15mar11 added section start ###################
+*STrigrid_S_imag3d=\&PDL::STrigrid_S_imag3d;
+sub PDL::STrigrid_S_imag3d { &checkargs;
+  &graph_object(new PDL::Graphics::TriD::STrigrid_S(@_)); }
+        
+*STrigrid_imag3d=\&PDL::STrigrid_imag3d;
+sub PDL::STrigrid_imag3d { &checkargs;
+  &graph_object(new PDL::Graphics::TriD::STrigrid(@_)); }
+################ JNK 15mar11 added section finis ###################
+####################################################################
+
 *mesh3d=\&PDL::mesh3d;
 *lattice3d=\&PDL::mesh3d;
 *PDL::lattice3d=\&PDL::mesh3d;
@@ -668,6 +880,12 @@ sub PDL::points3d { &checkargs;
 	&graph_object(new PDL::Graphics::TriD::Points(@_));
 }
 
+
+*spheres3d=\&PDL::spheres3d;
+sub PDL::spheres3d { &checkargs;
+	&graph_object(new PDL::Graphics::TriD::Spheres(@_));
+}
+
 *grabpic3d=\&PDL::grabpic3d;
 sub PDL::grabpic3d {
 	my $win = PDL::Graphics::TriD::get_current_window();
@@ -679,31 +897,38 @@ sub PDL::grabpic3d {
 
 $PDL::Graphics::TriD::hold_on = 0;
 
-sub PDL::hold3d {$PDL::Graphics::TriD::hold_on =
-			(!defined $_[0] ? 1 : $_[0]);}
+sub PDL::hold3d {$PDL::Graphics::TriD::hold_on =(!defined $_[0] ? 1 : $_[0]);}
 sub PDL::release3d {$PDL::Graphics::TriD::hold_on = 0;}
+
 *hold3d=\&PDL::hold3d;
 *release3d=\&PDL::release3d;
 
 sub get_new_graph {
+    print "get_new_graph: calling PDL::Graphics::TriD::get_current_window...\n" if($PDL::debug_trid);
 	my $win = PDL::Graphics::TriD::get_current_window();
-	my $g = get_current_graph();
+
+    print "get_new_graph: calling get_current_graph...\n" if($PDL::debug_trid);
+	my $g = get_current_graph($win);
+    print "get_new_graph: back get_current_graph returned $g...\n" if($PDL::debug_trid);
+
 	if(!$PDL::Graphics::TriD::hold_on) {
 		$g->clear_data();
+		$win->clear_viewport();
 	}
 	$g->default_axes();
-	$win->clear_viewports();
-	$win->clear_objects();
+
 	$win->add_object($g);
 	return $g;
 }
 
 sub get_current_graph {
-	my $g = $PDL::Graphics::TriD::curgraph;
+   my $win = shift;
+	my $g = $win->current_viewport()->graph();
+
 	if(!defined $g) {
 		$g = new PDL::Graphics::TriD::Graph();
 		$g->default_axes();
-		$PDL::Graphics::TriD::curgraph = $g;
+		$win->current_viewport()->graph($g);
 	}
 	return $g;
 }
@@ -712,39 +937,27 @@ sub get_current_graph {
 # $PDL::Graphics::TriD::cur = {};
 # $PDL::Graphics::TriD::create_window_sub = undef;
 sub get_current_window {
-	my $win = $PDL::Graphics::TriD::cur;
-	if(!defined $win->{Window}) {
-		if(!$PDL::Graphics::TriD::create_window_sub) {
-			barf("PDL::Graphics::TriD must be used with a display mechanism: for example PDL::Graphics::TriD::GL!\n");
-		}
-		$win->{Window} = & $PDL::Graphics::TriD::create_window_sub();
-		if ($win->{Window}->{Interactive}) {
-       		  require PDL::Graphics::TriD::ArcBall;
-		  require PDL::Graphics::TriD::SimpleScaler;
-		  require PDL::Graphics::TriD::Control3D;
-   		  $win->{EventHandler} = new PDL::Graphics::TriD::EventHandler();
-		  $win->{Window}->set_eventhandler($win->{EventHandler});
-		  $win->{Control} = new PDL::Graphics::TriD::SimpleController();
-		  $win->{ArcBall1} = new PDL::Graphics::TriD::ArcCone(
-			$win->{Window}, 0,
-			$win->{Control}{WRotation});
-		  $win->{Scaler} = new PDL::Graphics::TriD::SimpleScaler(
-			$win->{Window},
-			\$win->{Control}{CDistance});
-		  $win->{EventHandler}->set_button(0,$win->{ArcBall1});
-		  $win->{EventHandler}->set_button(2,$win->{Scaler});
-		  $win->{Window}->set_transformer($win->{Control});
-		}
-		$win->{Window}->set_material(new PDL::Graphics::TriD::Material);
-		$PDL::Graphics::TriD::current_window = $win->{Window};
-		$win->{MainWindow} = $win->{Window};
-		$PDL::Graphics::TriD::cur = $win
-	}
-	return $PDL::Graphics::TriD::current_window;
+  my $opts = shift @_;
+  my $win = $PDL::Graphics::TriD::cur;
+
+  if(!defined $win) {
+	 if(!$PDL::Graphics::TriD::create_window_sub) {
+		barf("PDL::Graphics::TriD must be used with a display mechanism: for example PDL::Graphics::TriD::GL!\n");
+	 }
+	 print "get_current_window - creating window...\n" if($PDL::debug_trid);
+	 $win = new PDL::Graphics::TriD::Window($opts);
+
+	 print "get_current_window - calling set_material...\n" if($PDL::debug_trid);
+	 $win->set_material(new PDL::Graphics::TriD::Material);
+	 $PDL::Graphics::TriD::current_window = $win;
+	 $PDL::Graphics::TriD::cur = $win
+  }
+  return $PDL::Graphics::TriD::current_window;
 }
 
 # Get the current graphbox
-sub get_current_graph {
+sub get_current_graphbox {
+        die "get_current_graphbox: ERROR graphbox is not implemented! \n";
 	my $graph = $PDL::Graphics::TriD::curgraph;
 	if(!defined $graph) {
 		$graph = new PDL::Graphics::TriD::Graph();
@@ -782,10 +995,13 @@ sub new {
 }
 
 package PDL::Graphics::TriD::BoundingBox;
-@ISA=qw/PDL::Graphics::TriD::Object/;
+use base qw/PDL::Graphics::TriD::Object/;
+use fields qw/Box/;
 
-sub new { my($type,$x0,$y0,$z0,$x1,$y1,$z1) = @_;
-	bless {Box => [$x0,$y0,$z0,$x1,$y1,$z1]},$type;
+sub new { 
+  my($type,$x0,$y0,$z0,$x1,$y1,$z1) = @_;
+  my $this = $type->SUPER::new();
+  $this->{Box} = [$x0,$y0,$z0,$x1,$y1,$z1];
 }
 
 sub normalize {my($this,$x0,$y0,$z0,$x1,$y1,$z1) = @_;
@@ -808,118 +1024,32 @@ sub normalize {my($this,$x0,$y0,$z0,$x1,$y1,$z1) = @_;
 ###################################
 #
 #
-package PDL::Graphics::TriD::Object;
-
-sub clear_objects {
-	my($this) = @_;
-	$this->{Objects} = [];
-	$this->{ValidList} = 0;
-}
-
-# XXXXXXXXX sub {} makes all these objects and this window immortal!
-sub add_object {
-	my($this,$object) = @_;
-	push @{$this->{Objects}},$object;
-	$this->{ValidList} = 0;
-	for(@{$this->{ChangedSub}}) {
-		$object->add_changedsub($_);
-	}
-	if($this->i_keep_list) {
-		$object->add_changedsub(sub {$this->changed_from_above()});
-	}
-}
-
-sub changed_from_above {
-	my($this) = @_;
-	print "CHANGED_FROM_ABOVE\n" if $PDL::Graphics::TriD::verbose;
-	$this->changed();
-}
-
-sub add_changedsub {
-	my($this,$chsub) = @_;
-	push @{$this->{ChangedSub}}, $chsub;
-	for (@{$this->{Objects}}) {
-		$_->add_changedsub($chsub);
-	}
-}
-
-
-sub clear {
-	my($this) = @_;
-	# print "Clear: $this\n";
-	for(@{$this->{Objects}}) {
-		$_->clear();
-	}
-	$this->delete_displist();
-	delete $this->{ChangedSub};
-	delete $this->{Objects};
-}
-
-sub changed {
-	my($this) = @_;
-	print "VALID0 $this\n" if $PDL::Graphics::TriD::verbose;
-	$this->{ValidList} = 0;
-	for(@{$this->{ChangedSub}}) {
-		&$_($this);
-	}
-}
-
-sub i_keep_list {
-	return 0;
-}
-
-##################################
-#
-# Does nothing by itself except makes an orthogonal window transformation
-# to the provided size elsewhere.
-#
-package PDL::Graphics::TriD::ViewPort;
-@ISA = qw/PDL::Graphics::TriD::Object/;
-sub i_keep_list {return 1} # For object.
+package PDL::Graphics::TriD::OneTransformation;
+use fields qw/Args/;
 
 sub new {
-	my($type) = @_;
-	my $this = bless {},$type;
-	# print "CREATE VP $this\n";
-	return $this;
+  my($type,@args) = @_;
+  my $this = fields::new($type);
+
+  $this->{Args} = [@args];
+
+  $this;
 }
 
-sub new_viewport {
-	my($this,$x0,$y0,$x1,$y1) = @_;
-	push @{$this->{ViewPorts}},[(new PDL::Graphics::TriD::ViewPort()),
-		$x0,$y0,$x1,$y1];
-	return $this->{ViewPorts}[-1][0];
-}
-
-sub DESTROY {
-	my($this) = @_;
-	# print "DESTROY VP $this\n";
-}
-
-###################################
-#
-#
 package PDL::Graphics::TriD::Scale;
-@ISA = qw/PDL::Graphics::TriD::OneTransformation/;
+use base qw/PDL::Graphics::TriD::OneTransformation/;
 
 package PDL::Graphics::TriD::Translation;
-@ISA = qw/PDL::Graphics::TriD::OneTransformation/;
+use base qw/PDL::Graphics::TriD::OneTransformation/;
 
-package PDL::Graphics::TriD::OneTransformation;
-
-sub new {
-	my($type,@args) = @_;
-	my $this = {Args => [@args]};
-	bless $this,$type;
-}
 
 package PDL::Graphics::TriD::Transformation;
-@ISA = qw/PDL::Graphics::TriD::Object/;
+use base qw/PDL::Graphics::TriD::Object/;
 
-sub new {
-	my($type) = @_;
-	bless {},$type;
-}
+#sub new {
+#	my($type) = @_;
+#	bless {},$type;
+#}
 
 sub add_transformation {
 	my($this,$trans) = @_;
@@ -927,8 +1057,6 @@ sub add_transformation {
 }
 
 
-package PDL::Graphics::TriD::Window;
-@ISA = qw/PDL::Graphics::TriD::Object/;
 
 =head1 BUGS
 
