@@ -27,15 +27,16 @@ PDL::CallExt - call functions in external shared libraries
 
 =head1 DESCRIPTION
 
-callext() loads in a shareable object (i.e. compiled code) using Perl's
-dynamic loader, calls the named function and passes a list of piddle arguments
-to it.
+callext() loads in a shareable object (i.e. compiled code) using
+Perl's dynamic loader, calls the named function and passes a list of
+piddle arguments to it.
 
-It provides a reasonably portable way of doing this, including compiling the
-code with the right flags, though it requires simple perl and C wrapper
-routines to be written. You may prefer to use PP, which is much more
-portable. See L<PDL::PP>. You should definitely use the latter for a 'proper'
-PDL module, or if you run in to the limitations of this module.
+It provides a reasonably portable way of doing this, including
+compiling the code with the right flags, though it requires simple
+perl and C wrapper routines to be written. You may prefer to use PP,
+which is much more portable. See L<PDL::PP>. You should definitely use
+the latter for a 'proper' PDL module, or if you run in to the
+limitations of this module.
 
 =head1 API
 
@@ -70,11 +71,11 @@ pdlsimple.h defines a simple N-dimensional data structure which looks like this:
 
 (PDL_Long is always a 4 byte int and is defined in pdlsimple.h)
 
-This is a simplification of the internal reprensation of piddles in PDL which is
+This is a simplification of the internal representation of piddles in PDL which is
 more complicated because of threading, dataflow, etc. It will usually be found
 somewhere like /usr/local/lib/perl5/site_perl/PDL/pdlsimple.h
 
-Thus to actually use this to call real functions one would need to right a wrapper.
+Thus to actually use this to call real functions one would need to write a wrapper.
 e.g. to call a 2D image processing routine:
 
  void myimage_processer(double* image, int nx, int ny);
@@ -190,35 +191,35 @@ sub callext_cc {
 	my $eo = ( $^O =~ /MSWin/i ? '"' : '' );
 
 	# Compiler command
+        # Placing $ccflags *before* installsitearch/PDL/Core enables us to include
+        # the blib 'pdlsimple.h' during 'make test'.
 	my $cc_cmd = join(' ', map { $Config{$_} } qw(cc ccflags cccdlflags)) .
-		" -I$Config{installsitelib}/PDL/Core $ccflags -c $src $do$cc_obj$eo";
+		qq{ $ccflags "-I$Config{installsitearch}/PDL/Core" -c $src $do$cc_obj$eo};
 
 	# The linker output flag is -o on cc and gcc, and -out: on MS Visual Studio
 	my $o = ( $Config{cc} eq 'cl' ? '-out:' : '-o ');
 
 	# Setup the LD command. Do not want the env var on Windows
-	my $ld_cmd = ( $^O =~ /MSWin/i ? ' ' : 'LD_RUN_PATH="" ');
+	my $ld_cmd = ( $^O =~ /MSWin|android/i ? ' ' : 'LD_RUN_PATH="" ');
 
-	my $libs = $^O =~ /MSWin/i ? 
+	my $libs = $^O =~ /MSWin/i ?
                  $Config{libs} :
                  ''; # used to be $Config{libs} but that bombs
 	               # on recent debian platforms
 	$ld_cmd .=
 		join(' ', map { $Config{$_} } qw(ld lddlflags)) .
 		" $libs $ldflags $o$ld_obj $cc_obj";
-	my $cmd = "$cc_cmd; $ld_cmd";
-	print $cmd,"\n";
 
 	# Run the command in two steps so that we can check status
 	# of each and also so that we dont have to rely on ';' command
 	# separator
 
-	system $cc_cmd and croak " compiling $src";
+	system $cc_cmd and croak "Error compiling $src ($cc_cmd)";
 
       # Fix up ActiveState-built perl. Is this a reliable fix ?
       $ld_cmd =~ s/\-nodefaultlib//g if $Config{cc} eq 'cl';
 
-	system $ld_cmd and croak "Error linking $cc_obj";
+	system $ld_cmd and croak "Error linking $cc_obj ($ld_cmd)";
 	return 1;
 }
 
